package edu.mtech.esof427.keyboard;

public class Rectangle extends Shape
{
    private double width  = 0.0;
    private double height = 0.0;

    public Rectangle(Point center, double width, double height)
    {
        super(center);
        this.width = width;
        this.height = height;
    }
    
    public double getWidth()
    {
        return width;
    }
    
    public double getHeight()
    {
        return height;
    }
    
    public double getLeft()
    {
        Point center = getCenter();
        return center.getX() - width / 2.0;        
    }

    public double getRight()
    {
        Point center = getCenter();
        return center.getX() + width / 2.0;        
    }
    
    public double getTop()
    {
        Point center = getCenter();
        return center.getY() - height / 2.0;        
    }
    
    public double getBottom()
    {
        Point center = getCenter();
        return center.getY() + height / 2.0;        
    }
    
    public boolean isInside(Point point)
    {
        double x = point.getX();
        double y = point.getY();
        
        return (x >= getLeft()  && 
                x <= getRight() && 
                y >= getTop()   && 
                y <= getBottom());        
    }
    
    // Bounding box of a rectangle is easy, it is the same as ourselves!
    public Rectangle getBoundingBox()
    {
        return new Rectangle(getCenter(), width, height);
    }
    
    // Check these two boxes for intersection
    public boolean intersects(Rectangle other)
    {
        double p1_x = this.getLeft();
        double p1_y = this.getBottom();
        double p2_x = this.getRight();
        double p2_y = this.getTop();
        
        double p3_x = other.getLeft();
        double p3_y = other.getBottom();
        double p4_x = other.getRight();
        double p4_y = other.getTop();

        return (p2_y <= p3_y && p1_y >= p4_y && p2_x >= p3_x && p1_x <= p4_x);
    }
    
    public String toString()
    {
        return String.format("[RECTANGLE %s w=%.2f h=%.2f]", getCenter().toString(), width, height);
    }

}
