package edu.mtech.esof427.keyboard;

import java.io.*;
import java.util.*;

public class RectangleKeyboard
{
    private ArrayList<RectangleKey> keys = new ArrayList<RectangleKey>();

    public RectangleKeyboard(String filename) throws IOException
    {   
        BufferedReader reader = new BufferedReader(new FileReader(filename));
        String line = reader.readLine();        
        while (line != null)
        {
            line = line.trim();

            // Lines starting with # are comments we ignore
            if (line.indexOf('#') != 0)
            {
                RectangleKey key = new RectangleKey(line);
                keys.add(key);
            }            
            line = reader.readLine();
        }
        reader.close();
    }

    // Return the recognized text given a sequence of taps.
    // If focalMode = 0, we take the average of the locations in the trace
    // If focalMode = 1, we use the first touch location in the trace
    // If focalMode = 2, we use the last touch location in the trace
    // If mustBeInside is true, then the trace must be inside a key- if not we return null.
    public String getText(TraceSequence sequence, int focalMode, boolean mustBeInside)
    {
        StringBuilder sb = new StringBuilder();
        
        for (Trace t : sequence)
        {
            String s = getCharacter(t, focalMode, mustBeInside);
            if (s != null)
                sb.append(s);
        }
        return sb.toString();
    }
    
    // Return the character of the key based on the trace.
    private String getCharacter(Trace trace, int focalMode, boolean mustBeInside)
    {
        Point point = null;
        String result = null;

        if (focalMode == 0)
            point = (new ComputeTraceAvg()).getFocalPoint(trace);
        else if (focalMode == 1)
            point = (new ComputeTraceFirst()).getFocalPoint(trace);
        else if (focalMode == 2)
            point = (new ComputeTraceLast()).getFocalPoint(trace);

        if (mustBeInside)
        {
            for (RectangleKey key : keys)
            {
                if (key.isInside(point))
                    return key.getCharacter();
            }
        }
        else
        {
            double minDistance = Double.POSITIVE_INFINITY;
            for (RectangleKey key : keys)
            {
                double distance = key.getDistanceToCenter(point);
                if (distance < minDistance)
                {
                    minDistance = distance;
                    result = key.getCharacter();
                }
            }           
        }        
        return result;
    }
}
