package edu.mtech.esof427.keyboard;

import java.util.*;

public abstract class Trace 
{
    private List<Touch> touches = new ArrayList<Touch>();

    // Add a touch to the collection
    public boolean add(Touch touch)
    {
        return touches.add(touch);
    }
        
    public int size()
    {
        return touches.size();
    }
    
    public Touch get(int index)
    {
        return touches.get(index);
    }

    // Return a rectangle that bounds the points in our collection of touches
    public Rectangle getBoundingBox()
    {
        double minX = Double.POSITIVE_INFINITY;
        double maxX = Double.NEGATIVE_INFINITY;
        double minY = Double.POSITIVE_INFINITY;
        double maxY = Double.NEGATIVE_INFINITY;
        
        for (Touch touch: touches)
        {
            Point location = touch.getLocation();
            double x = location.getX();
            double y = location.getY();

            minX = Math.min(minX, x);
            minY = Math.min(minY, y);
            
            maxX = Math.max(maxX, x);
            maxY = Math.max(maxY, y);
        }
        double width = maxX - minX;
        double height = maxY - minY;
        
        double centerX = width  / 2.0 + minX;
        double centerY = height / 2.0 + minY;

        Point center = new Point(centerX, centerY);        
        return new Rectangle(center, width, height);        
    }

    public String toString()
    {
        StringBuffer result = new StringBuffer();
        
        int i = 0;
        for (Touch touch : touches)
        {
            result.append(String.format("%4d : ", i));
            result.append(touch.toString());
            result.append("\n");
            i++;
        }
        
        return result.toString();
    }

}
