package edu.mtech.esof427.keyboard.test;

import static org.junit.Assert.*;

import org.junit.Test;

import edu.mtech.esof427.keyboard.*;

public class CircleTest
{
    private static final double EPSILON = 1e-8;
    
    @Test
    public void testX()
    {
        Circle circle = new Circle(new Point(5.0, 10.0), 3.0);        
        assertEquals(5.0, circle.getCenter().getX(), EPSILON);        

        Circle circle2 = new Circle(new Point(-5.0, -10.0), 3.0);        
        assertEquals(-5.0, circle2.getCenter().getX(), EPSILON);        

    }

    @Test
    public void testY()
    {
        Circle circle = new Circle(new Point(5.0, 10.0), 3.0);        
        assertEquals(10.0, circle.getCenter().getY(), EPSILON);        

        Circle circle2 = new Circle(new Point(-5.0, -10.0), 3.0);        
        assertEquals(-10.0, circle2.getCenter().getY(), EPSILON);    
    }

    @Test
    public void testRadius()
    {
        Circle circle = new Circle(new Point(5.0, 10.0), 3.0);        
        assertEquals(3.0, circle.getRadius(), EPSILON);        
    }

    @Test
    public void testInside()
    {
        Circle circle = new Circle(new Point(5.0, 10.0), 3.0);        

        for (int i = 0; i < 100000; i++)
        {
            double x = Math.random() * 20.0 - 10.0;
            double y = Math.random() * 20.0 - 10.0;
            Point p = new Point(x, y);
            
            double deltaX = x - 5.0;
            double deltaY = y - 10.0;
            double dist = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
            
            if (dist <= 3.0)
                assertTrue(circle.isInside(p));
            else
                assertFalse(circle.isInside(p));            
        }
    }
    
    @Test
    public void testBoundingBox()
    {
        Circle circle = new Circle(new Point(5.0, 10.0), 3.0);        
        Rectangle bound = circle.getBoundingBox();
        
        assertEquals(3.0 * 2, bound.getWidth(), EPSILON);
        assertEquals(3.0 * 2, bound.getHeight(), EPSILON);
        
        Point center = bound.getCenter();
        assertEquals(5.0, center.getX(), EPSILON);
        assertEquals(10.0, center.getY(), EPSILON);
    }
    
    @Test
    public void testToString()
    { 
        Circle circle = new Circle(new Point(5.0, 10.0), 3.0); 
        String str = circle.toString();
        
        assertTrue(str.length() > 0);
    }
    
}
