package edu.mtech.esof427.keyboard.test;

import static org.junit.Assert.*;

import org.junit.Test;

import edu.mtech.esof427.keyboard.Point;
import edu.mtech.esof427.keyboard.Rectangle;
import edu.mtech.esof427.keyboard.ScreenOrientation;
import edu.mtech.esof427.keyboard.Touch;
import edu.mtech.esof427.keyboard.TouchTime;
import edu.mtech.esof427.keyboard.TraceOrientation;

public class TraceOrientationTest
{
    private final static double EPSILON = 1e-8;
    
    @Test
    public void testGetOrientation()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        
        assertEquals(ScreenOrientation.Portrait, trace.getOrientation());
        assertNotEquals(ScreenOrientation.Landscape, trace.getOrientation());
        assertNotEquals(ScreenOrientation.PortraitUpsideDown, trace.getOrientation());
        assertNotEquals(ScreenOrientation.LandscapeUpsideDown, trace.getOrientation());        
    }
    
    @Test
    public void testToString()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);  
        TouchTime time = new TouchTime(new Point(0, 0), 0);
        trace.add(time);
        
        String str = trace.toString();
        assertTrue(str.length() > 0);
    }

    @Test
    public void testAddTouches()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = 0; i < 100; i++)
        {
            assertEquals(i, trace.size());
            TouchTime time = new TouchTime(new Point(i, i + 1), i + 2);
            trace.add(time);

            assertEquals(i + 1, trace.size());
            
            Touch time2 = trace.get(i);
            assertEquals(time.getLocation().getX(), time2.getLocation().getX(), EPSILON);
            assertEquals(time.getLocation().getY(), time2.getLocation().getY(), EPSILON);
        }
    }
    
    @Test
    public void testBoundingBoxDiagonalLine()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchTime time = new TouchTime(new Point(i, i), 0);
            trace.add(time);
        }
        
        Rectangle bound = trace.getBoundingBox();
        assertEquals(-100.0, bound.getLeft(), EPSILON);
        assertEquals(100.0, bound.getRight(), EPSILON);
        assertEquals(-100.0, bound.getTop(), EPSILON);
        assertEquals(100.0, bound.getBottom(), EPSILON);               
    }

    @Test
    public void testBoundingBoxHorizontalLine()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchTime time = new TouchTime(new Point(i, 5.0), 0);
            trace.add(time);
        }
        
        Rectangle bound = trace.getBoundingBox();
        assertEquals(-100.0, bound.getLeft(), EPSILON);
        assertEquals(100.0, bound.getRight(), EPSILON);
        assertEquals(5.0, bound.getTop(), EPSILON);
        assertEquals(5.0, bound.getBottom(), EPSILON);               
    }

    @Test
    public void testBoundingBoxVerticalLine()
    {
        TraceOrientation trace = new TraceOrientation(ScreenOrientation.Portrait);        

        for (int i = -100; i <= 100; i++)
        {
            TouchTime time = new TouchTime(new Point(5.0, i), 0);
            trace.add(time);
        }
        
        Rectangle bound = trace.getBoundingBox();
        assertEquals(5.0, bound.getLeft(), EPSILON);
        assertEquals(5.0, bound.getRight(), EPSILON);
        assertEquals(-100.0, bound.getTop(), EPSILON);
        assertEquals(100.0, bound.getBottom(), EPSILON);               
    }
}
